require 'i18n'

# Hax to load missing constants due to bug in i18n gem
require 'i18n/core_ext/hash'

module AionRequestTranslations

  REQUEST_STORE_KEY = :request_translations

  # An i18n backend that stores a list of i18n backends on the request.
  # This makes it possible to load translations in a thread safe manner
  # that cleans up after the request.
  class Backend < I18n::Backend::Chain

    # Adds a i18n backend to the beginning of the chain of backends.
    # @return [I18n::Backend::Base] an array of i18n backends
    def self.add_backend(backend)
      RequestStore.fetch(REQUEST_STORE_KEY) { [] }.unshift(backend)
    end

    # Adds a i18n backend with the given translations.
    # Translations should be a hash with locale as key.
    #
    # @param translations [Hash] a hash with locale as keys
    # @return [I18n::Backend::Base] an array of i18n backends
    def self.add_translations(translations)
      backend = I18n::Backend::KeyValue.new({}, true)
      translations.each { |locale, data|
        backend.store_translations(locale, data || {})
      }
      add_backend(backend)
    end

    def backends
      RequestStore[REQUEST_STORE_KEY] ||= []
    end

    def backends=(backends)
      RequestStore[REQUEST_STORE_KEY] = backends
    end

  end

end
